@@ -21,21 +21,13 @@
 #
 # ---------------------------------------------------------------------------##
 
-# imports
-
-# PySol imports
-from pysollib.gamedb import registerGame, GameInfo, GI
-from pysollib.mfxutil import kwdefault
 from pysollib.game import Game
-from pysollib.layout import Layout
+from pysollib.gamedb import GI, GameInfo, registerGame
+from pysollib.games.spider import Spider_Hint, Spider_RowStack, \
+        Spider_SS_Foundation
 from pysollib.hint import KlondikeType_Hint, YukonType_Hint
-
-from pysollib.games.spider import Spider_SS_Foundation, \
-        Spider_RowStack, Spider_Hint
-
-from pysollib.util import ACE, ANY_SUIT, KING, UNLIMITED_ACCEPTS, \
-        UNLIMITED_MOVES
-
+from pysollib.layout import Layout
+from pysollib.mfxutil import kwdefault
 from pysollib.stack import \
         AC_RowStack, \
         AbstractFoundationStack, \
@@ -49,17 +41,19 @@ from pysollib.stack import \
         SS_RowStack, \
         Spider_SS_RowStack, \
         Stack, \
+        StackWrapper, \
         TalonStack, \
         WasteStack, \
         WasteTalonStack, \
-        Yukon_AC_RowStack, \
-        StackWrapper
-
+        Yukon_AC_RowStack
+from pysollib.util import ACE, ANY_SUIT, KING, UNLIMITED_ACCEPTS, \
+        UNLIMITED_MOVES
 
 # ************************************************************************
 # * Gypsy
 # ************************************************************************
 
+
 class Gypsy(Game):
     Layout_Method = staticmethod(Layout.gypsyLayout)
     Talon_Class = DealRowTalonStack
@@ -879,6 +873,17 @@ class TopsyTurvyQueens(LockedCards):
     shallHighlightMatch = Game._shallHighlightMatch_SSW
 
 
+class KingsSecrets(LockedCards):
+    Foundation_Class = StackWrapper(LockedCards_Foundation,
+                                    base_rank=KING, mod=13)
+    RowStack_Class = StackWrapper(SS_RowStack, mod=13, base_rank=11)
+
+    def startGame(self):
+        LockedCards.startGame(self, rows=5)
+
+    shallHighlightMatch = Game._shallHighlightMatch_SSW
+
+
 # ************************************************************************
 # * Thirty
 # ************************************************************************
@@ -1027,3 +1032,5 @@ registerGame(GameInfo(721, Thirty, "Thirty",
                       ranks=(0, 6, 7, 8, 9, 10, 11, 12)))
 registerGame(GameInfo(725, TopsyTurvyQueens, "Topsy-Turvy Queens",
                       GI.GT_2DECK_TYPE, 2, 2, GI.SL_BALANCED))
+registerGame(GameInfo(792, KingsSecrets, "King's Secrets",
+                      GI.GT_2DECK_TYPE, 2, 2, GI.SL_BALANCED))
