@@ -152,8 +152,11 @@ class SelectCardsetData(SelectDialogTreeData):
                     None, _("Large cardsets"),
                     lambda cs: cs.si.size == CSI.SIZE_LARGE),
                  SelectCardsetNode(
-                    None, _("XLarge cardsets"),
+                    None, _("Extra Large cardsets"),
                     lambda cs: cs.si.size == CSI.SIZE_XLARGE),
+                 SelectCardsetNode(
+                    None, _("Hi-Res cardsets"),
+                    lambda cs: cs.si.size == CSI.SIZE_HIRES),
                  ), expanded=1),
             select_by_type,
             select_by_style,
@@ -223,11 +226,15 @@ class SelectCardsetDialogWithPreview(MfxDialog):
         if self.TreeDataHolder_Class.data is None:
             self.TreeDataHolder_Class.data = self.TreeData_Class(manager, key)
         #
+        sw = self.top.winfo_screenwidth()
+        sh = self.top.winfo_screenheight()
+
+        h = int(sh * .8)
+        w = int(sw * .8)
+        w1 = int(min(275, sw / 2.5))
+        geometry = ("%dx%d+%d+%d" % (w, h, (sw - w) / 2, (sh - h) / 2))
         self.top.wm_minsize(400, 200)
-        if self.top.winfo_screenwidth() >= 800:
-            w1, w2 = 240, 400
-        else:
-            w1, w2 = 200, 300
+
         paned_window = ttk.PanedWindow(top_frame, orient='horizontal')
         paned_window.pack(expand=True, fill='both')
         left_frame = ttk.Frame(paned_window)
@@ -264,6 +271,30 @@ class SelectCardsetDialogWithPreview(MfxDialog):
             self.scale_y.grid(
                 row=2, column=0, sticky='ew', padx=padx, pady=pady)
             #
+            # sliders at new position
+            cs = self.manager.get(self.tree.selection_key)
+
+            var = tkinter.IntVar()
+            self.x_offset = PysolScale(
+                left_frame, label=_('X offset:'),
+                from_=5, to=100, resolution=1,
+                orient='horizontal', variable=var,
+                value=cs.CARD_XOFFSET
+                )
+
+            self.x_offset.grid(row=3, column=0, sticky='ew',
+                               padx=padx, pady=pady)
+
+            var = tkinter.IntVar()
+            self.y_offset = PysolScale(
+                left_frame, label=_('Y offset:'),
+                from_=5, to=100, resolution=1,
+                orient='horizontal', variable=var,
+                value=cs.CARD_YOFFSET
+                )
+            self.y_offset.grid(row=4, column=0, sticky='ew',
+                               padx=padx, pady=pady)
+
             self.auto_scale = tkinter.BooleanVar()
             self.auto_scale.set(app.opt.auto_scale)
             check = ttk.Checkbutton(
@@ -272,7 +303,7 @@ class SelectCardsetDialogWithPreview(MfxDialog):
                 takefocus=False,
                 command=self._updateAutoScale
                 )
-            check.grid(row=3, column=0, columnspan=2, sticky='ew',
+            check.grid(row=5, column=0, columnspan=2, sticky='ew',
                        padx=padx, pady=pady)
             #
             self.preserve_aspect = tkinter.BooleanVar()
@@ -283,14 +314,15 @@ class SelectCardsetDialogWithPreview(MfxDialog):
                 takefocus=False,
                 # command=self._updateScale
                 )
-            self.aspect_check.grid(row=4, column=0, sticky='ew',
+            self.aspect_check.grid(row=6, column=0, sticky='ew',
                                    padx=padx, pady=pady)
+
             self._updateAutoScale()
         #
         left_frame.rowconfigure(0, weight=1)
         left_frame.columnconfigure(0, weight=1)
         #
-        self.preview = MfxScrolledCanvas(right_frame, width=w2)
+        self.preview = MfxScrolledCanvas(right_frame)
         self.preview.setTile(app, app.tabletile_index, force=True)
         self.preview.pack(fill='both', expand=True, padx=padx, pady=pady)
         self.preview.canvas.preview = 1
@@ -302,7 +334,7 @@ class SelectCardsetDialogWithPreview(MfxDialog):
         #
         focus = self.createButtons(bottom_frame, kw)
         focus = self.tree.frame
-        self.mainloop(focus, kw.timeout)
+        self.mainloop(focus, kw.timeout, geometry=geometry)
 
     def destroy(self):
         self.tree.updateNodesWithTree(self.tree.rootnodes, None)
@@ -312,10 +344,7 @@ class SelectCardsetDialogWithPreview(MfxDialog):
         MfxDialog.destroy(self)
 
     def initKw(self, kw):
-        if USE_PIL:
-            s = (_("&Info / Settings..."), 10)
-        else:
-            s = (_("&Info..."), 10)
+        s = (_("&Info..."), 10)
         kw = KwStruct(kw,
                       strings=(s, 'sep',
                                _("&OK"), _("&Cancel"),),
@@ -328,20 +357,49 @@ class SelectCardsetDialogWithPreview(MfxDialog):
         if button in (0, 1):            # Load/Cancel
             self.key = self.tree.selection_key
             self.tree.n_expansions = 1  # save xyview in any case
+
+            if button == 0:
+                cardset = self.app.cardset_manager.get(self.key)
+                if self.app.game is not None:
+                    gi = self.app.getGameInfo(self.app.game.id)
+                else:
+                    gi = self.app.getGameInfo(self.app.nextgame.id)
+                cs, cs_update_flag, t = \
+                    self.app.getCompatibleCardset(gi, cardset, trychange=False)
+
+                if cs is None:
+                    self.app.requestCompatibleCardsetTypeDialog(cardset, gi, t)
+                    return
+
+            # save the values
+            try:
+                self.cardset_values = self.x_offset.get(), self.y_offset.get()
+            except Exception:
+                pass
+
             if USE_PIL:
                 auto_scale = bool(self.auto_scale.get())
                 if button == 1:
+                    # no changes
+                    self.cardset_values = None
+
+                elif button == 0:
                     self.app.menubar.tkopt.auto_scale.set(auto_scale)
-                if auto_scale:
-                    self.scale_values = (self.app.opt.scale_x,
-                                         self.app.opt.scale_y,
-                                         auto_scale,
-                                         bool(self.preserve_aspect.get()))
-                else:
-                    self.scale_values = (self.scale_x.get(),
-                                         self.scale_y.get(),
-                                         auto_scale,
-                                         self.app.opt.preserve_aspect_ratio)
+
+                    if auto_scale:
+                        self.app.menubar.tkopt.spread_stacks.set(False)
+                        self.scale_values = (self.app.opt.scale_x,
+                                             self.app.opt.scale_y,
+                                             auto_scale,
+                                             False,
+                                             bool(self.preserve_aspect.get()))
+                    else:
+                        self.scale_values = (self.scale_x.get(),
+                                             self.scale_y.get(),
+                                             auto_scale,
+                                             self.app.opt.spread_stacks,
+                                             self.app.opt.
+                                             preserve_aspect_ratio)
         if button == 10:                # Info
             cs = self.manager.get(self.tree.selection_key)
             if not cs:
@@ -350,7 +408,11 @@ class SelectCardsetDialogWithPreview(MfxDialog):
             title = CARDSET.capitalize()+" "+cs.name
             d = CardsetInfoDialog(self.top, title=title, cardset=cs,
                                   images=self.preview_images)
-            self.cardset_values = d.cardset_values
+            try:
+                self.cardset_values = d.cardset_values
+            except Exception:
+                self.cardset_values = None
+
             return
         MfxDialog.mDone(self, button)
 
@@ -380,6 +442,15 @@ class SelectCardsetDialogWithPreview(MfxDialog):
             self.preview_key = -1
             return
         names, columns = cs.getPreviewCardNames()
+
+        # if cardset has changed, set default values
+        if key != self.preview_key and USE_PIL:
+            self.x_offset.config(value=cs.CARD_XOFFSET)
+            self.x_offset.set(cs.CARD_XOFFSET)
+
+            self.y_offset.config(value=cs.CARD_YOFFSET)
+            self.y_offset.set(cs.CARD_YOFFSET)
+
         try:
             # ???names, columns = cs.getPreviewCardNames()
             for n in names:
@@ -482,39 +553,13 @@ class CardsetInfoDialog(MfxDialog):
                 info_frame.rowconfigure(frow, weight=1)
             except Exception:
                 pass
-        if USE_PIL:
-            padx = 4
-            pady = 0
-            settings_frame = ttk.LabelFrame(frame, text=_('Settings'))
-            settings_frame.grid(row=row, column=0, columnspan=2, sticky='ew',
-                                padx=0, pady=5, ipadx=5, ipady=5)
-            row += 1
-            var = tkinter.IntVar()
-            self.x_offset = PysolScale(
-                settings_frame, label=_('X offset:'),
-                from_=5, to=40, resolution=1,
-                orient='horizontal', variable=var,
-                value=cardset.CARD_XOFFSET,
-                # command=self._updateScale
-                )
-            self.x_offset.grid(row=0, column=0, sticky='ew',
-                               padx=padx, pady=pady)
-            var = tkinter.IntVar()
-            self.y_offset = PysolScale(
-                settings_frame, label=_('Y offset:'),
-                from_=5, to=40, resolution=1,
-                orient='horizontal', variable=var,
-                value=cardset.CARD_YOFFSET,
-                # command=self._updateScale
-                )
-            self.y_offset.grid(row=1, column=0, sticky='ew',
-                               padx=padx, pady=pady)
+
             row += 1
 
         # bg = top_frame["bg"]
         bg = 'white'
         text_w = tkinter.Text(frame, bd=1, relief="sunken", wrap="word",
-                              padx=4, width=64, height=16, bg=bg)
+                              padx=4, width=64, height=8, bg=bg)
         text_w.grid(row=row, column=0, sticky='nsew')
         sb = ttk.Scrollbar(frame)
         sb.grid(row=row, column=1, sticky='ns')
@@ -524,9 +569,10 @@ class CardsetInfoDialog(MfxDialog):
         frame.rowconfigure(1, weight=1)
         #
         text = ''
-        f = os.path.join(cardset.dir, "COPYRIGHT")
+        fn = os.path.join(cardset.dir, "COPYRIGHT")
         try:
-            text = open(f).read()
+            with open(fn, "rt") as fh:
+                text = fh.read()
         except Exception:
             pass
         if text:
@@ -537,23 +583,3 @@ class CardsetInfoDialog(MfxDialog):
         focus = self.createButtons(bottom_frame, kw)
         # focus = text_w
         self.mainloop(focus, kw.timeout)
-
-    def initKw(self, kw):
-        if USE_PIL:
-            strings = (_("&Save"), _("&Cancel"))
-        else:
-            strings = (_("&OK"),)
-        kw = KwStruct(kw,
-                      strings=strings,
-                      default=0,
-                      resizable=True,
-                      separator=True,
-                      )
-        return MfxDialog.initKw(self, kw)
-
-    def mDone(self, button):
-        if USE_PIL and button == 0:
-            self.cardset_values = self.x_offset.get(), self.y_offset.get()
-        else:
-            self.cardset_values = None
-        MfxDialog.mDone(self, button)
