@@ -24,16 +24,17 @@
 import os
 import traceback
 
-from pysollib.mygettext import _
-from pysollib.settings import TITLE
-from pysollib.settings import VERSION
-from pysollib.settings import TOOLKIT, USE_TILE
-from pysollib.settings import DEBUG
 from pysollib.mfxutil import print_err
+from pysollib.mygettext import _
+from pysollib.settings import DEBUG
+from pysollib.settings import TITLE
+from pysollib.settings import TOOLKIT, USE_TILE
+from pysollib.settings import VERSION
 
 if TOOLKIT == 'tk':
+    from pysollib.ui.tktile.tkutil import loadImage
     if USE_TILE:
-        from pysollib.tile import ttk
+        from six.moves import tkinter_ttk as ttk
 
 
 def init_tile(app, top):
@@ -89,6 +90,16 @@ def base_init_root_window(root, app):
     # root.wm_group(root)
     root.wm_title(TITLE + ' ' + VERSION)
     root.wm_iconname(TITLE + ' ' + VERSION)
+
+    if TOOLKIT == 'tk':
+        icons = [loadImage(img) for img in app.dataloader.findAllIconSizes()]
+        icons.sort(reverse=True, key=lambda img: img.width())
+        if icons:
+            try:
+                root.wm_iconphoto(True, *icons)
+            except AttributeError:
+                root.tk.call('wm', 'iconphoto', root, '-default', *icons)
+
     # set minsize
     sw, sh = (root.winfo_screenwidth(), root.winfo_screenheight())
     if sw < 640 or sh < 480:
@@ -96,16 +107,10 @@ def base_init_root_window(root, app):
     else:
         root.wm_minsize(520, 360)
 
-    if TOOLKIT == 'gtk':
-        pass
-    if TOOLKIT == 'kivy':
-        pass
-    elif USE_TILE:
+    if TOOLKIT == 'tk' and USE_TILE:
         theme = app.opt.tile_theme
         init_tile(app, root)
         set_theme(app, root, theme)
-    else:
-        pass
 
 
 class BaseTkSettings:
